package com.renderx.xepx.x4u.ct;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import org.w3c.dom.Document;
import org.w3c.dom.*;
import java.util.*;
/**
 * <p>Title: BackendPanel</p>
 * <p>Description: UI for BackEnd parameters </p>
* <p>Copyright: Copyright (c) 2004</p>
* <p>Company: Zenteq.am</p>
* @author: Boris Karapetyan
* @version: 0.5
* @todo: 1. Add "New Backend" command with option table (name-value paars)
*/

public class BackendPanel extends JPanel implements ActionListener {
  Document doc;
  DOMWalker model;
//  Node pdfNode = null;
//  Node psNode = null;
  Element[] bkNode = new Element[DOMWalker.BACKEND_COUNT];
  // ComboBox values
  static final String[] ZOOM_VALUE = {
      "auto", "fit-screen", "fit-width", "fit-height", "number-or-percentage "};
  static final String[] VIEW_VALUE = {
      "auto","show-none", "show-bookmarks", "show-thumbnails", "full-screen"};
  static final String[] PRIVEL_VALUE = {
      "annotate", "copy", "modify", "print"};
//  static String[] backends; //  current {"PDF","PostScript"};
  JComboBox selectorComboBox; // filled from *.xml (see initValuesFromDOM())
  JLabel selectorLabel = new JLabel("Select backend");
  JPanel selectorPanel = new JPanel();
  Border border = new EtchedBorder(EtchedBorder.RAISED, Color.white, Color.darkGray); //new Color(148, 145, 140));
  JPanel parametersPanel = new JPanel();
  JPanel customParamPanel = new JPanel();
  // Common params panel with UI Elements
  JPanel commonParamPanel = new JPanel();
  JCheckBox dropCheckBox = new JCheckBox("Drop unused destination", true); //new JCheckBox();
  JCheckBox unicodeCheckBox = new JCheckBox("UNICODE annotations", true); //new JCheckBox();
  JComboBox zoomComboBox = new JComboBox(ZOOM_VALUE);
  JComboBox viewComboBox = new JComboBox(VIEW_VALUE);
  JLabel viewLabel = new JLabel("Set Inital view mode");
  JLabel zoomLabel = new JLabel("Set inital zoom value");
  JTextField zoomTextField = new JTextField();

  //  pdf options
  JPanel pdfPanel = new JPanel();
  JCheckBox ownerPSWCheckBox = new JCheckBox("Set owner password", false);
  JTextField ownerPSWTextField = new JTextField();
  JCheckBox userPSWCheckBox = new JCheckBox("Set user password", false);
  JTextField userPSWTextField = new JTextField();
  JLabel privLabel = new JLabel("User Privileges");
  JComboBox privComboBox = new JComboBox(PRIVEL_VALUE);
  JCheckBox compressCheckBox = new JCheckBox("Use PDF compression", true);
  JCheckBox linearizeCheckBox = new JCheckBox("Use PDF linearization", false);

  //  post script options
  JPanel psPanel = new JPanel();
  JCheckBox cloneCheckBox = new JCheckBox("Clone EPS images", true);
  JComboBox levelComboBox = new JComboBox(new String[] {"3", "2"});
  JLabel levelLabel = new JLabel("Select PS Level");

  public BackendPanel(Document doc) { //int selected, Document doc) {
    this.doc = doc;
    model = DOMWalker.getDOMWalker(doc);////////////new DOMWalker(doc);
/////    initValuesFromDOM();
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    /////if (this.backends.length > 0)
    setValuesFromDOM(bkNode[selectorComboBox.getSelectedIndex()]);
  }

/*  private void initValuesFromDOM() {
    java.util.List temp = new ArrayList();
    NodeList nl = model.optRoot.getElementsByTagName(DOMWalker.BACKEND_TAG);
    if (nl == null) this.backends = new String[0];
    else{
      for (int i = 0; i < nl.getLength(); i++) {
        if (i < DOMWalker.BACKEND_COUNT) bkNode[i] = nl.item(i);
        Element nd = (Element) nl.item(i);
        if (nd.getNodeName().equals(DOMWalker.BACKEND_TAG)) {
          temp.add(nd.getAttribute("format"));
        }
      }
      this.backends = (String[]) temp.toArray(new String[temp.size()]);
    }
  }*/

  private void jbInit() throws Exception {
    this.setLayout(new BorderLayout());
    parametersPanel.setBorder(new TitledBorder(border, "Backend Parameters"));
    parametersPanel.setLayout(new BorderLayout());
    selectorComboBox = new JComboBox(model.BACKEND_NAMES);
    selectorComboBox.setActionCommand("selectorComboBox");
    selectorComboBox.addActionListener(this);
    selectorComboBox.setSelectedIndex(0);
    selectorPanel.setBorder(border);
    zoomComboBox.setActionCommand("zoomCombo");
    viewLabel.setText("Set inital view mode");
    linearizeCheckBox.addActionListener(this);
    linearizeCheckBox.setActionCommand("linearizeCheckBox");
    linearizeCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);

    zoomTextField.setEnabled(false);
    zoomTextField.setEditable(false);
    zoomTextField.setText(" 100 ");
    zoomTextField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        actionPerformed(new ActionEvent(e.getSource(),e.getID(),"zoomTextField"));
      }
    });
    zoomTextField.setActionCommand("zoomTextField");
    selectorPanel.add(selectorLabel, null);
    selectorPanel.add(selectorComboBox, null);
    // UI Elements for Common params
    dropCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    dropCheckBox.setActionCommand("Dropdest");
    dropCheckBox.addActionListener(this);

    unicodeCheckBox.setActionCommand("UNICODE");
    unicodeCheckBox.addActionListener(this);
    unicodeCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);

    viewComboBox.setActionCommand("viewComboBox");
    viewComboBox.addActionListener(this);
    zoomComboBox.setActionCommand("zoomComboBox");
    zoomComboBox.addActionListener(this);

//    zoomComboBox.setName("Select initial box");
    commonParamPanel.setLayout(new GridBagLayout());
    commonParamPanel.add(zoomLabel,    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
    commonParamPanel.add(zoomComboBox,
                             new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));
    commonParamPanel.add(viewLabel,   new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
    commonParamPanel.add(viewComboBox,
                           new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));
    commonParamPanel.add(dropCheckBox,
                            new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
    commonParamPanel.add(unicodeCheckBox,
                            new GridBagConstraints(2, 0, 1, 2, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    commonParamPanel.add(zoomTextField,       new GridBagConstraints(3, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));

    // UI Elements for PDF-specific params
    pdfPanel.setLayout(new GridBagLayout()); //gridBagLayout2);

    ownerPSWCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    ownerPSWCheckBox.setActionCommand("ownerPSWCheckBox");
    ownerPSWCheckBox.addActionListener(this);

    ownerPSWTextField.setEnabled(false);
    ownerPSWTextField.setPreferredSize(new Dimension(100, 21));
    ownerPSWTextField.setEditable(false);
    ownerPSWTextField.addActionListener(this);
    ownerPSWTextField.setActionCommand("ownerPSWTextField");
    ownerPSWTextField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        actionPerformed(new ActionEvent(e.getSource(),e.getID(),"ownerPSWTextField"));
      }
    });

    userPSWCheckBox.setHorizontalAlignment(SwingConstants.RIGHT);
    userPSWCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    userPSWTextField.setEditable(false);
    userPSWTextField.setActionCommand("userPSWTextField");
    userPSWTextField.addActionListener(this);
    userPSWTextField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        actionPerformed(new ActionEvent(e.getSource(),e.getID(),"userPSWTextField"));
      }
    });

    userPSWTextField.setPreferredSize(new Dimension(100, 21));
    userPSWTextField.setEnabled(false);
    userPSWCheckBox.setActionCommand("userPSWCheckBox");
    userPSWCheckBox.addActionListener(this);

    privLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    privComboBox.setActionCommand("privCombo");
    privComboBox.addActionListener(this);
    privComboBox.setEnabled(false);

    compressCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    compressCheckBox.setActionCommand("compressCheckBox");
    compressCheckBox.addActionListener(this);

    pdfPanel.add(ownerPSWCheckBox,     new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 1, 0));
    pdfPanel.add(ownerPSWTextField,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 31, 2));
    pdfPanel.add(userPSWCheckBox,     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
    pdfPanel.add(userPSWTextField,   new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 31, 2));
    pdfPanel.add(privLabel,     new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
    pdfPanel.add(privComboBox,   new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
    pdfPanel.add(compressCheckBox,    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    pdfPanel.add(linearizeCheckBox,    new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    customParamPanel.add(pdfPanel, null); //////--> selectorComboBox action

    // UI Elements for PS-specific params
    psPanel.setLayout(new GridBagLayout()); //gridBagLayout3);
    cloneCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    cloneCheckBox.setActionCommand("cloneCheckBox");
    cloneCheckBox.addActionListener(this);

    levelLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    levelComboBox.setActionCommand("levelComboBox");
    levelComboBox.addActionListener(this);

    this.add(selectorPanel, BorderLayout.NORTH);
    this.add(parametersPanel, BorderLayout.CENTER);
    parametersPanel.add(customParamPanel, BorderLayout.CENTER);
//////////customParamPanel.add(psPanel, null);
    parametersPanel.add(commonParamPanel, BorderLayout.NORTH);
    psPanel.add(levelComboBox, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
        new Insets(0, 0, 0, 5), 85, 2));
    psPanel.add(cloneCheckBox, new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));
    psPanel.add(levelLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 5), 30, 8));
    selectorComboBox.setSelectedIndex(0);
  }

  public void actionPerformed(ActionEvent e) {
    String action = e.getActionCommand();
    int ind = selectorComboBox.getSelectedIndex();
    if (action.equals("selectorComboBox")) {
      customParamPanel.removeAll();
      NodeList tmp = model.optRoot.getElementsByTagName("generator-options"); //doc.getElementsByTagName("generator-options");
      Element bk = null;
      if (tmp != null){
        for (int i = 0; i < tmp.getLength(); i++) {
          Element nd = (Element)tmp.item(i);
          if (nd.getAttributes().getNamedItem("format").getNodeValue().
              equalsIgnoreCase(selectorComboBox.getSelectedItem().toString())){
            bk = nd;
            break;
          }
        }
      }
      if(bk == null){
        bk = doc.createElement("generator-options");
        bk.setAttribute("format", selectorComboBox.getItemAt(ind).toString());
        DOMWalker.optRoot.appendChild(bk);
      }
      if (bkNode[ind] == null)
        bkNode[ind] = bk;
      switch (ind) {
        case 0:
          customParamPanel.add(pdfPanel, null);
//          if (pdfNode == null) pdfNode = nd;
          break;
        case 1:
          customParamPanel.add(psPanel, null);
//          if (psNode == null) psNode = nd;
          break;
      }
      setValuesFromDOM(bkNode[selectorComboBox.getSelectedIndex()]);
      customParamPanel.updateUI();
    }
    else {
      if (action.equals("Dropdest"))
        model.setNameValueAttrs(bkNode[ind],"option","DROP_UNUSED_DESTINATIONS",""+dropCheckBox.isSelected());
      else if (action.equals("UNICODE"))
        model.setNameValueAttrs(bkNode[ind],"option","UNICODE_ANNOTATIONS",""+unicodeCheckBox.isSelected());
      else if (action.equals("viewComboBox"))
        model.setNameValueAttrs(bkNode[ind],"option","VIEW_MODE", viewComboBox.getSelectedItem().toString());
      else if (action.equals("zoomTextField"))
        model.setNameValueAttrs(bkNode[ind],"option","INITIAL_ZOOM", zoomTextField.getText().trim());
      else if (action.equals("zoomComboBox")){
        if (zoomComboBox.getSelectedIndex() == (ZOOM_VALUE.length - 1)){
          zoomTextField.setEditable(true);
          zoomTextField.setEnabled(true);
        }
        else {
          model.setNameValueAttrs(bkNode[ind],"option","INITIAL_ZOOM", zoomComboBox.getSelectedItem().toString());
          zoomTextField.setEditable(false);
          zoomTextField.setEnabled(false);
        }
      }
      else if (action.equals("ownerPSWTextField"))
        model.setNameValueAttrs(bkNode[ind],"option","OWNERPASSWORD", ownerPSWTextField.getText().trim());
      else if (action.equals("userPSWTextField"))
        model.setNameValueAttrs(bkNode[ind],"option","USERPASSWORD", userPSWTextField.getText().trim());
      else if (action.equals("privCombo"))
        model.setNameValueAttrs(bkNode[ind],"option","USERPRIVILEGES", privComboBox.getSelectedItem().toString());
      else if (action.equals("compressCheckBox"))
        model.setNameValueAttrs(bkNode[ind],"option","COMPRESS", ""+compressCheckBox.isSelected());
      else if (action.equals("linearizeCheckBox"))
        model.setNameValueAttrs(bkNode[ind],"option","LINEARIZE", ""+linearizeCheckBox.isSelected());
      else if (action.equals("ownerPSWCheckBox")) {
        ownerPSWTextField.setEnabled(ownerPSWCheckBox.isSelected());
        ownerPSWTextField.setEditable(ownerPSWCheckBox.isSelected());
        if(! ownerPSWCheckBox.isSelected())
          model.removeNode(bkNode[ind],"option","name","OWNERPASSWORD");
       }
      else if (action.equals("userPSWCheckBox")) {
        userPSWTextField.setEnabled(userPSWCheckBox.isSelected());
        userPSWTextField.setEditable(userPSWCheckBox.isSelected());
        privComboBox.setEnabled(userPSWCheckBox.isSelected());
        if(! userPSWCheckBox.isSelected()){
          model.removeNode(bkNode[ind], "option", "name", "USERPASSWORD");
          model.removeNode(bkNode[ind], "option", "name", "USERPRIVILEGES");
        }
      }
      ////// PostScript options levelComboBox
      else if (action.equals("levelComboBox"))
        model.setNameValueAttrs(bkNode[ind],"option","LANGUAGE_LEVEL", levelComboBox.getSelectedItem().toString());
      else if (action.equals("cloneCheckBox"))
        model.setNameValueAttrs(bkNode[ind],"option","CLONE_EPS",""+cloneCheckBox.isSelected());
    }
  }



  void setValuesFromDOM(Node bRoot){
    NodeList lst = bRoot.getChildNodes();
    for (int i = 0; i < lst.getLength(); i++) {
      if(lst.item(i).getNodeName().equalsIgnoreCase("option")){
        String name  = lst.item(i).getAttributes().getNamedItem("name").getNodeValue();
        String value = lst.item(i).getAttributes().getNamedItem("value").getNodeValue();
        if (value == null) continue;
//////////System.out.println("name="+name+"; value="+value);
        boolean state = (value.length() > 0);
        if(name.equalsIgnoreCase("OWNERPASSWORD")){
            ownerPSWCheckBox.setSelected(state);
            ownerPSWTextField.setText(value);
            ownerPSWTextField.setEnabled(state);
            ownerPSWTextField.setEditable(state);
        }
        else if(name.equalsIgnoreCase("USERPASSWORD")){
          userPSWCheckBox.setSelected(state);
          userPSWTextField.setText(value);
          userPSWTextField.setEnabled(state);
          userPSWTextField.setEditable(state);
          privComboBox.setEnabled(state);
        }
        else if(name.equalsIgnoreCase("USERPRIVILEGES")){
          privComboBox.setSelectedItem(value);
        }
        else if(name.equalsIgnoreCase("COMPRESS")){
          compressCheckBox.setSelected(state && value.equalsIgnoreCase("true"));
        }
        else if(name.equalsIgnoreCase("LINEARIZE")){
          linearizeCheckBox.setSelected(state && value.equalsIgnoreCase("true"));
        }
        else if(name.equalsIgnoreCase("UNICODE_ANNOTATIONS")){
        unicodeCheckBox.setSelected(state && value.equalsIgnoreCase("true"));}
        else if(name.equalsIgnoreCase("DROP_UNUSED_DESTINATIONS")){
        dropCheckBox.setSelected(state && value.equalsIgnoreCase("true"));}
        else if(name.equalsIgnoreCase("INITIAL_ZOOM")){
          if(! Arrays.asList(ZOOM_VALUE).contains(value)){
            zoomTextField.setText(" "+value+" ");
            zoomTextField.setEditable(true);
            zoomTextField.setEnabled(true);
            zoomComboBox.setSelectedIndex(ZOOM_VALUE.length - 1);
          }
          else zoomComboBox.setSelectedItem(value);
        }
        else if(name.equalsIgnoreCase("VIEW_MODE")){
          viewComboBox.setSelectedItem(value);
        }
        else if(name.equalsIgnoreCase("LANGUAGE_LEVEL")){
          levelComboBox.setSelectedItem(value);
        }
        else if(name.equalsIgnoreCase("CLONE_EPS")){
          cloneCheckBox.setSelected(value.equalsIgnoreCase("true"));
        }
      }
    }
  }
}

